-- RelicInspector.lua
-- Author: Thoralie

local addonName, addon = ...
local RelicSpells = addon.SpellIDByRelic

local DEBUG = 0

local options, optionsFrame, db

local modifierPressed = 0

local RelicTypeNames = {
	["Blood"] = RELIC_SLOT_TYPE_BLOOD,
	["Shadow"] = RELIC_SLOT_TYPE_SHADOW,
	["Iron"] = RELIC_SLOT_TYPE_IRON,
	["Frost"] = RELIC_SLOT_TYPE_FROST,
	["Fire"] = RELIC_SLOT_TYPE_FIRE,
	["Fel"] = RELIC_SLOT_TYPE_FEL,
	["Arcane"] = RELIC_SLOT_TYPE_ARCANE,
	["Life"] = RELIC_SLOT_TYPE_LIFE,
	["Wind"] = RELIC_SLOT_TYPE_WIND,
	["Holy"] = RELIC_SLOT_TYPE_HOLY,
}

local RelicSlotsByArtifact = {
	[127829] = {"Fel","Shadow","Fel"}, -- Havoc DH
	[128832] = {"Iron","Arcane","Fel"}, -- Vengeance DH

	[128402] = {"Blood","Shadow","Iron"}, -- Blood DK
	[128292] = {"Frost","Shadow","Frost"}, -- Frost DK
	[128403] = {"Fire","Shadow","Blood"}, -- Unholy DK

	[128858] = {"Arcane","Life","Arcane"}, -- Balance Druid
	[128860] = {"Frost","Blood","Life"}, -- Feral Druid
	[128821] = {"Fire","Blood","Life"}, -- Guardian Druid
	[128306] = {"Life","Frost","Life"}, -- Restoration Druid

	[128861] = {"Wind","Arcane","Iron"}, -- Beast Mastery Hunter
	[128826] = {"Wind","Blood","Life"}, -- Marksmanship Hunter
	[128808] = {"Wind","Iron","Blood"}, -- Survival Hunter

	[127857] = {"Arcane","Frost","Arcane"}, -- Arcane Mage
	[128820] = {"Fire","Arcane","Fire"}, -- Fire Mage
	[128862] = {"Frost","Arcane","Frost"}, -- Frost Mage

	[128938] = {"Life","Wind","Iron"}, -- Brewmaster Monk
	[128937] = {"Frost","Life","Wind"}, -- Mistweaver Monk
	[128940] = {"Wind","Iron","Wind"}, -- Windwalker Monk

	[128823] = {"Holy","Life","Holy"}, -- Holy Paladin
	[128866] = {"Holy","Iron","Arcane"}, -- Protection Paladin
	[120978] = {"Holy","Fire","Holy"}, -- Retribution Paladin

	[128868] = {"Holy","Shadow","Holy"}, -- Discipline Priest
	[128825] = {"Holy","Life","Holy"}, -- Holy Priest
	[128827] = {"Shadow","Blood","Shadow"}, -- Shadow Priest

	[128870] = {"Shadow","Iron","Blood"}, -- Assassination Rogue
	[128872] = {"Blood","Iron","Wind"}, -- Outlaw Rogue
	[128476] = {"Fel","Shadow","Fel"}, -- Subtlety Rogue

	[128935] = {"Wind","Frost","Wind"}, -- Elemental Shaman
	[128819] = {"Fire","Iron","Wind"}, -- Enhancement Shaman
	[128911] = {"Life","Frost","Life"}, -- Restoration Shaman

	[128942] = {"Shadow","Blood","Shadow"}, -- Affliction Warlock
	[128943] = {"Shadow","Fire","Fel"}, -- Demonology Warlock
	[128941] = {"Fel","Fire","Fel"}, -- Destruction Warlock

	[128910] = {"Iron","Blood","Shadow"}, -- Arms Warrior
	[128908] = {"Fire","Wind","Iron"}, -- Fury Warrior
	[128289] = {"Iron","Blood","Fire"}, -- Protection Warrior
}

local SpecByArtifact = {
	[127829] = 577, -- Havoc DH
	[128832] = 581, -- Vengeance DH

	[128402] = 250, -- Blood DK
	[128292] = 251, -- Frost DK
	[128403] = 252, -- Unholy DK

	[128858] = 102, -- Balance Druid
	[128860] = 103, -- Feral Druid
	[128821] = 104, -- Guardian Druid
	[128306] = 105, -- Restoration Druid

	[128861] = 253, -- Beast Mastery Hunter
	[128826] = 254, -- Marksmanship Hunter
	[128808] = 255, -- Survival Hunter

	[127857] = 62, -- Arcane Mage
	[128820] = 63, -- Fire Mage
	[128862] = 64, -- Frost Mage

	[128938] = 268, -- Brewmaster Monk
	[128937] = 270, -- Mistweaver Monk
	[128940] = 269, -- Windwalker Monk

	[128823] = 65, -- Holy Paladin
	[128866] = 66, -- Protection Paladin
	[120978] = 70, -- Retribution Paladin

	[128868] = 256, -- Discipline Priest
	[128825] = 257, -- Holy Priest
	[128827] = 258, -- Shadow Priest

	[128870] = 259, -- Assassination Rogue
	[128872] = 260, -- Outlaw Rogue
	[128476] = 261, -- Subtlety Rogue

	[128935] = 262, -- Elemental Shaman
	[128819] = 263, -- Enhancement Shaman
	[128911] = 264, -- Restoration Shaman

	[128942] = 265, -- Affliction Warlock
	[128943] = 266, -- Demonology Warlock
	[128941] = 267, -- Destruction Warlock

	[128910] = 71, -- Arms Warrior
	[128908] = 72, -- Fury Warrior
	[128289] = 73, -- Protection Warrior	
}

local hoverOptions = {
    [1] = "Never",
    [2] = "CTRL Key",
    [3] = "Always" 
}

local invHoverOptions = {
	["Never"] = 1,
	["CTRL Key"] = 2,
    ["Always"] = 3
}

local locHoverOptions = {
    [1] = NEVER,
    [2] = CTRL_KEY,
    [3] = ALWAYS
}

local linkOptions = {
    [1] = "Never",
    [2] = "Always" 
}

local invLinkOptions = {
	["Never"] = 1,
    ["Always"] = 2
}

local locLinkOptions = {
    [1] = NEVER,
    [2] = ALWAYS
}

local defaults = {
	profile = {
		enabled = true,
		hoverTraitNames = "Always",
		hoverTraitDesc = "CTRL Key",
		linkTraitNames = "Always",
		linkTraitDesc = "Always",
		showSpecs = true,
	}
}

local function SetupOptions()
	if options then return options end

	options = {
		name = "RelicInspector",
		type = "group",
		args = {
			general = {
				name = "Options",
				type = "group",
				args = {
					enabled = {
						name = "Enable",
						desc = "Enables / Disables RelicInspector",
						type = "toggle",
						set = function(info,val) db.profile.enabled = val end,
						get = function(info) return db.profile.enabled end,
						order = 10
					},
					relicSpecs = {
						name = "Show Specs on Relics",
						desc = "Enables / Disables display of player specs that can use a relic",
						type = "toggle",
						set = function(info,val) db.profile.showSpecs = val end,
						get = function(info) return db.profile.showSpecs end,
						order = 15
					},
					itemTooltipHeader = {
						name = "Item Tooltip Options",
						type = "header",
						order = 20
					},
					itemTooltipTraitNames = {
						name = "Show Trait Names:",
						desc = "When to show trait names on item tooltips",
						type = "select",
						width = "single",
						set = function(info,val) db.profile.hoverTraitNames = hoverOptions[val] end,
						get = function(info) return invHoverOptions[db.profile.hoverTraitNames] end,
						order = 30,
						values = locHoverOptions
					},
					itemTooltipTraitDesc = {
						name = "Show Trait Descriptions:",
						desc = "When to show trait descriptions on item tooltips",
						type = "select",
						width = "single",
						set = function(info,val) db.profile.hoverTraitDesc = hoverOptions[val] end,
						get = function(info) return invHoverOptions[db.profile.hoverTraitDesc] end,
						order = 40,
						values = locHoverOptions
					},
					linkTooltipHeader = {
						name = "Link Tooltip Options",
						type = "header",
						order = 50
					},
					linkTooltipTraitNames = {
						name = "Show Trait Names:",
						desc = "When to show trait names on link tooltips",
						type = "select",
						width = "single",
						set = function(info,val) db.profile.linkTraitNames = linkOptions[val] end,
						get = function(info) return invLinkOptions[db.profile.linkTraitNames] end,
						order = 60,
						values = locLinkOptions
					},
					linkTooltipTraitDesc = {
						name = "Show Trait Descriptions:",
						desc = "When to show trait descriptions on link tooltips",
						type = "select",
						width = "single",
						set = function(info,val) db.profile.linkTraitDesc = linkOptions[val] end,
						get = function(info) return invLinkOptions[db.profile.linkTraitDesc] end,
						order = 70,
						values = locLinkOptions
					},
				}
			},
		}
	}

	return options
end

local function initialize()
	db = _G.LibStub("AceDB-3.0"):New("RelicInspectorDB", defaults, "Default")
	local config = _G.LibStub("AceConfig-3.0")
	local dialog = _G.LibStub("AceConfigDialog-3.0")
	local options = SetupOptions()

	config:RegisterOptionsTable("RelicInspector", options)
	optionsFrame = dialog:AddToBlizOptions("RelicInspector", "RelicInspector", nil, "general")
end

local function DecorateTooltip(self)
	local _, link = self:GetItem()
	if type(link) == 'string' and db.profile.enabled == true then
		local _, itemID, _, relic1, relic2, relic3, _, _, _, _, _, upgradeID = strsplit(':', link)

		if nil == itemID or '' == itemID then return end  -- If there's no itemID we can't do anything

		if upgradeID == '256' and nil ~= RelicSlotsByArtifact[tonumber(itemID)] then
			--It's a recognized artifact item
			local relics = {relic1,relic2,relic3}
			for i = 1,3 do 
				if '' ~= relics[i] then
					local name, gemLink = GetItemGem(link,i)
					if type(gemLink) == 'string' then
						--Display the gem name, item level, and relic type
						local _, _, _, gemLevel = GetItemInfo(gemLink)
						local relicType = RelicTypeNames[RelicSlotsByArtifact[tonumber(itemID)][i]]

						if nil == relicType then
							relicType = "???"
						end
						self:AddDoubleLine(format('|cffffd400 (%s) %s|r', gemLevel or '???', gemLink or '???'), 
							format('|cff00ff00(%s)',relicType), 0, 1, 0, 0, 1, 0)

						-- Check options to see what else to display
						local showTraitNames, showTraitDesc
						if(self:GetName() == "ItemRefTooltip") then
							-- Item Links
							if invLinkOptions[db.profile.linkTraitNames] == 2 then
								showTraitNames = true
							else 
								showTraitNames = false
							end

							if invLinkOptions[db.profile.linkTraitDesc] == 2 then
								showTraitDesc = true
							else 
								showTraitDesc = false
							end
						else -- Game or Shopping Hover Tooltips
							if invHoverOptions[db.profile.hoverTraitNames] == 3 or
								(invHoverOptions[db.profile.hoverTraitNames] == 2 and modifierPressed ~= 0) then
								showTraitNames = true
							else 
								showTraitNames = false
							end

							if invHoverOptions[db.profile.hoverTraitDesc] == 3 or
								(invHoverOptions[db.profile.hoverTraitDesc] == 2 and modifierPressed ~= 0) then
								showTraitDesc = true
							else 
								showTraitDesc = false
							end
						end

						-- Let's add the Ranks and Trait names if we are supposed to
						if showTraitNames or showTraitDesc then
							local spellLookupKey = itemID .. '.' .. relics[i]
							local traitSpellID = RelicSpells[spellLookupKey]

							if nil ~= traitSpellID then
								--Add Trait Name
								if showTraitNames == true then
									local traitName = GetSpellInfo(traitSpellID)
									if nil ~= traitName then
										-- Assuming 1 Rank per relic right now. Will have to update this if relics start giving more than 1 rank
										self:AddLine(format(' ' .. RELIC_TOOLTIP_RANK_INCREASE, 1, traitName), 1, 1, 1, false)
									end
								end
								--Add Trait Description
								if showTraitDesc == true then
									local traitDesc = GetSpellDescription(traitSpellID)
									if nil ~= traitDesc then
										self:AddLine(format('|cffff78ff  %s|r', traitDesc), 1, 1, 1, true)
									end
								end
							end
						end
					end	
				end
			end
		else 
			if db.profile.showSpecs and ((self:GetName() == "ItemRefTooltip") or (self:GetName() == "GameTooltip")) then
				if DEBUG ~=0 then print(format('ItemID: %s',itemID)) end
				local _, _, relicTypeKey = C_ArtifactUI.GetRelicInfoByItemID(tonumber(itemID))
				-- Display Class Specs on Relics (not very useful in shopping tooltips)
				if nil ~= relicTypeKey then	
					if DEBUG ~=0 then print(format('RelicKey: %s',relicTypeKey)) end
					local usableSpecs = {}
					for k, v in pairs(RelicSlotsByArtifact) do
						local hasFound = false
						for i = 1,3 do
							if(v[i] == relicTypeKey) and hasFound == false then
								local aSpec = SpecByArtifact[k]
								if DEBUG ~=0 then print(format('Spec Found: %d',aSpec)) end
								tinsert(usableSpecs,aSpec)
								hasFound = true
							end
						end
					end
					table.sort(usableSpecs) -- Sort it by specID to keep class specs together
					local finalString = "|cffffffff"..format(ITEM_REQ_SPECIALIZATION,'')
					for iSpec = 1, #usableSpecs do
	 					local _, specName, _, _, _, _, classKey = GetSpecializationInfoByID(usableSpecs[iSpec])
	 					local classColorSet = RAID_CLASS_COLORS[classKey]
	 					local classColor = classColorSet.colorStr
	 					local separator = ", "
	 					if iSpec == #usableSpecs then separator = "" end
	 					local specString = format('|c%s%s|r%s',classColor,specName,separator)
	 					finalString = finalString .. specString
					end
					self:AddLine(format('%s', finalString), 1, 1, 1, true)
				end
			end
		end 
	end
end

GameTooltip:RegisterEvent("MODIFIER_STATE_CHANGED")
GameTooltip:HookScript("OnEvent", function(self, event, key, state)
	if event == "MODIFIER_STATE_CHANGED" and (key == "LCTRL" or key == "RCTRL") then
		modifierPressed = state
	end
end)

local f = CreateFrame('frame')
f:SetScript('OnEvent', function(self, event, arg1)
  if arg1 == addonName then
    self:UnregisterEvent('ADDON_LOADED')
    initialize()
  end
end)
f:RegisterEvent('ADDON_LOADED')

ItemRefTooltip:HookScript('OnTooltipSetItem', DecorateTooltip)
GameTooltip:HookScript('OnTooltipSetItem', DecorateTooltip)
ShoppingTooltip1:HookScript('OnTooltipSetItem', DecorateTooltip)
ShoppingTooltip2:HookScript('OnTooltipSetItem', DecorateTooltip)

